@@ -277,12 +277,12 @@ GRN_API grn_command_version grn_get_default_command_version(void);
 GRN_API grn_rc grn_set_default_command_version(grn_command_version version);
 GRN_API grn_command_version grn_ctx_get_command_version(grn_ctx *ctx);
 GRN_API grn_rc grn_ctx_set_command_version(grn_ctx *ctx, grn_command_version version);
-GRN_API long long int grn_ctx_get_match_escalation_threshold(grn_ctx *ctx);
-GRN_API grn_rc grn_ctx_set_match_escalation_threshold(grn_ctx *ctx, long long int threshold);
+GRN_API int64_t grn_ctx_get_match_escalation_threshold(grn_ctx *ctx);
+GRN_API grn_rc grn_ctx_set_match_escalation_threshold(grn_ctx *ctx, int64_t threshold);
 GRN_API grn_bool grn_ctx_get_force_match_escalation(grn_ctx *ctx);
 GRN_API grn_rc grn_ctx_set_force_match_escalation(grn_ctx *ctx, grn_bool force);
-GRN_API long long int grn_get_default_match_escalation_threshold(void);
-GRN_API grn_rc grn_set_default_match_escalation_threshold(long long int threshold);
+GRN_API int64_t grn_get_default_match_escalation_threshold(void);
+GRN_API grn_rc grn_set_default_match_escalation_threshold(int64_t threshold);
 
 GRN_API grn_rc grn_ctx_set_variable(grn_ctx *ctx,
                                     const char *name,
@@ -572,7 +572,9 @@ typedef enum {
   GRN_OP_IN,
   GRN_OP_MATCH,
   GRN_OP_NEAR,
-  GRN_OP_NEAR2,
+  GRN_OP_NEAR_NO_OFFSET,
+  /* Deprecated. Just for backward compatibility. */
+#define GRN_OP_NEAR2 GRN_OP_NEAR_NO_OFFSET
   GRN_OP_SIMILAR,
   GRN_OP_TERM_EXTRACT,
   GRN_OP_SHIFTL,
@@ -616,51 +618,13 @@ typedef enum {
   GRN_OP_QUORUM,
   GRN_OP_NEAR_PHRASE,
   GRN_OP_ORDERED_NEAR_PHRASE,
+  GRN_OP_NEAR_PHRASE_PRODUCT,
+  GRN_OP_ORDERED_NEAR_PHRASE_PRODUCT,
 } grn_operator;
 
 GRN_API grn_obj *grn_obj_column(grn_ctx *ctx, grn_obj *table,
                                 const char *name, unsigned int name_size);
 
-/*-------------------------------------------------------------
- * API for column
- */
-
-#define GRN_COLUMN_NAME_ID            "_id"
-#define GRN_COLUMN_NAME_ID_LEN        (sizeof(GRN_COLUMN_NAME_ID) - 1)
-#define GRN_COLUMN_NAME_KEY           "_key"
-#define GRN_COLUMN_NAME_KEY_LEN       (sizeof(GRN_COLUMN_NAME_KEY) - 1)
-#define GRN_COLUMN_NAME_VALUE         "_value"
-#define GRN_COLUMN_NAME_VALUE_LEN     (sizeof(GRN_COLUMN_NAME_VALUE) - 1)
-#define GRN_COLUMN_NAME_SCORE         "_score"
-#define GRN_COLUMN_NAME_SCORE_LEN     (sizeof(GRN_COLUMN_NAME_SCORE) - 1)
-#define GRN_COLUMN_NAME_NSUBRECS      "_nsubrecs"
-#define GRN_COLUMN_NAME_NSUBRECS_LEN  (sizeof(GRN_COLUMN_NAME_NSUBRECS) - 1)
-#define GRN_COLUMN_NAME_MAX           "_max"
-#define GRN_COLUMN_NAME_MAX_LEN       (sizeof(GRN_COLUMN_NAME_MAX) - 1)
-#define GRN_COLUMN_NAME_MIN           "_min"
-#define GRN_COLUMN_NAME_MIN_LEN       (sizeof(GRN_COLUMN_NAME_MIN) - 1)
-#define GRN_COLUMN_NAME_SUM           "_sum"
-#define GRN_COLUMN_NAME_SUM_LEN       (sizeof(GRN_COLUMN_NAME_SUM) - 1)
-/* Deprecated since 10.0.4. Use GRN_COLUMN_NAME_MEAN instead. */
-#define GRN_COLUMN_NAME_AVG           "_avg"
-#define GRN_COLUMN_NAME_AVG_LEN       (sizeof(GRN_COLUMN_NAME_AVG) - 1)
-#define GRN_COLUMN_NAME_MEAN          "_mean"
-#define GRN_COLUMN_NAME_MEAN_LEN      (sizeof(GRN_COLUMN_NAME_MEAN) - 1)
-
-GRN_API grn_obj *grn_column_create(grn_ctx *ctx, grn_obj *table,
-                                   const char *name, unsigned int name_size,
-                                   const char *path, grn_column_flags flags, grn_obj *type);
-
-#define GRN_COLUMN_OPEN_OR_CREATE(ctx,table,name,name_size,path,flags,type,column) \
-  (((column) = grn_obj_column((ctx), (table), (name), (name_size))) ||\
-   ((column) = grn_column_create((ctx), (table), (name), (name_size), (path), (flags), (type))))
-
-GRN_API grn_rc grn_column_index_update(grn_ctx *ctx, grn_obj *column,
-                                       grn_id id, unsigned int section,
-                                       grn_obj *oldvalue, grn_obj *newvalue);
-GRN_API grn_obj *grn_column_table(grn_ctx *ctx, grn_obj *column);
-GRN_API grn_rc grn_column_truncate(grn_ctx *ctx, grn_obj *column);
-
 /*-------------------------------------------------------------
  * API for db, table and/or column
  */
@@ -704,7 +668,8 @@ typedef enum {
   GRN_INFO_SUPPORT_ZSTD,
 /* Just for backward compatibility. */
 #define GRN_INFO_SUPPORT_ARROW GRN_INFO_SUPPORT_APACHE_ARROW
-  GRN_INFO_SUPPORT_APACHE_ARROW
+  GRN_INFO_SUPPORT_APACHE_ARROW,
+  GRN_INFO_NORMALIZERS,
 } grn_info_type;
 
 GRN_API grn_obj *grn_obj_get_info(grn_ctx *ctx, grn_obj *obj, grn_info_type type, grn_obj *valuebuf);
@@ -813,6 +778,7 @@ GRN_API unsigned int grn_obj_is_locked(grn_ctx *ctx, grn_obj *obj);
 GRN_API grn_rc grn_obj_flush(grn_ctx *ctx, grn_obj *obj);
 GRN_API grn_rc grn_obj_flush_recursive(grn_ctx *ctx, grn_obj *obj);
 GRN_API grn_rc grn_obj_flush_recursive_dependent(grn_ctx *ctx, grn_obj *obj);
+GRN_API grn_rc grn_obj_flush_only_opened(grn_ctx *ctx, grn_obj *obj);
 GRN_API int grn_obj_defrag(grn_ctx *ctx, grn_obj *obj, int threshold);
 
 GRN_API grn_obj *grn_obj_db(grn_ctx *ctx, grn_obj *obj);
@@ -858,6 +824,9 @@ struct _grn_search_optarg {
   grn_match_info match_info;
   int quorum_threshold;
   int additional_last_interval;
+  float *weight_vector_float;
+  float weight_float;
+  grn_obj *query_options;
 };
 
 GRN_API grn_rc grn_obj_search(grn_ctx *ctx, grn_obj *obj, grn_obj *query,
@@ -989,6 +958,15 @@ GRN_API grn_rc grn_snip_add_cond(grn_ctx *ctx, grn_obj *snip,
 GRN_API grn_rc grn_snip_set_normalizer(grn_ctx *ctx, grn_obj *snip,
                                        grn_obj *normalizer);
 GRN_API grn_obj *grn_snip_get_normalizer(grn_ctx *ctx, grn_obj *snip);
+GRN_API grn_rc
+grn_snip_set_delimiter_regexp(grn_ctx *ctx,
+                              grn_obj *snip,
+                              const char *pattern,
+                              int pattern_length);
+GRN_API const char *
+grn_snip_get_delimiter_regexp(grn_ctx *ctx,
+                              grn_obj *snip,
+                              size_t *pattern_length);
 GRN_API grn_rc grn_snip_exec(grn_ctx *ctx, grn_obj *snip,
                              const char *string, unsigned int string_len,
                              unsigned int *nresults, unsigned int *max_tagged_len);
@@ -1159,7 +1137,7 @@ GRN_API off_t grn_default_query_logger_get_rotate_threshold_size(void);
 #define GRN_BULK_BUFSIZE (sizeof(grn_obj) - sizeof(grn_obj_header))
 /* This assumes that GRN_BULK_BUFSIZE is less than 32 (= 0x20). */
 #define GRN_BULK_BUFSIZE_MAX 0x1f
-#define GRN_BULK_SIZE_IN_FLAGS(flags) ((flags) & GRN_BULK_BUFSIZE_MAX)
+#define GRN_BULK_SIZE_IN_FLAGS(flags) (size_t)((flags) & GRN_BULK_BUFSIZE_MAX)
 #define GRN_BULK_OUTP(bulk) ((bulk)->header.impl_flags & GRN_OBJ_OUTPLACE)
 #define GRN_BULK_REWIND(bulk) do {\
   if ((bulk)->header.type == GRN_VECTOR) {\
@@ -1196,15 +1174,15 @@ GRN_API off_t grn_default_query_logger_get_rotate_threshold_size(void);
 } while (0)
 #define GRN_BULK_WSIZE(bulk) \
   (GRN_BULK_OUTP(bulk)\
-   ? ((bulk)->u.b.tail - (bulk)->u.b.head)\
+   ? (size_t)((bulk)->u.b.tail - (bulk)->u.b.head)\
    : GRN_BULK_BUFSIZE)
 #define GRN_BULK_REST(bulk) \
   (GRN_BULK_OUTP(bulk)\
-   ? ((bulk)->u.b.tail - (bulk)->u.b.curr)\
+   ? (size_t)((bulk)->u.b.tail - (bulk)->u.b.curr)\
    : GRN_BULK_BUFSIZE - GRN_BULK_SIZE_IN_FLAGS((bulk)->header.flags))
 #define GRN_BULK_VSIZE(bulk) \
   (GRN_BULK_OUTP(bulk)\
-   ? ((bulk)->u.b.curr - (bulk)->u.b.head)\
+   ? (size_t)((bulk)->u.b.curr - (bulk)->u.b.head) \
    : GRN_BULK_SIZE_IN_FLAGS((bulk)->header.flags))
 #define GRN_BULK_EMPTYP(bulk) \
   (GRN_BULK_OUTP(bulk)\
@@ -1233,6 +1211,7 @@ GRN_API grn_rc grn_bulk_write_from(grn_ctx *ctx, grn_obj *bulk,
 GRN_API grn_rc grn_bulk_reserve(grn_ctx *ctx, grn_obj *bulk, size_t len);
 GRN_API grn_rc grn_bulk_space(grn_ctx *ctx, grn_obj *bulk, size_t len);
 GRN_API grn_rc grn_bulk_truncate(grn_ctx *ctx, grn_obj *bulk, size_t len);
+GRN_API char *grn_bulk_detach(grn_ctx *ctx, grn_obj *bulk);
 GRN_API grn_rc grn_bulk_fin(grn_ctx *ctx, grn_obj *bulk);
 
 /* grn_text */
