@@ -1,6 +1,6 @@
 /*
   Copyright(C) 2009-2018 Brazil
-  Copyright(C) 2018 Kouhei Sutou <kou@clear-code.com>
+  Copyright(C) 2018-2020 Sutou Kouhei <kou@clear-code.com>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -31,6 +31,13 @@ typedef struct _grn_obj_format grn_obj_format;
    Use GRN_OBJ_FORMAT_AS_ARRAY instead.*/
 #define GRN_OBJ_FORMAT_ASARRAY             GRN_OBJ_FORMAT_AS_ARRAY
 #define GRN_OBJ_FORMAT_WITH_WEIGHT         (0x01<<4)
+/* Format weight as float32.
+   Since 10.0.3 */
+#define GRN_OBJ_FORMAT_WEIGHT_FLOAT32      (0x01<<5)
+/* Call grn_ctx_output_flush() internally for each 1024 records in a table.
+   The "1024" value may be changed.
+   Since 10.0.3 */
+#define GRN_OBJ_FORMAT_AUTO_FLUSH          (0x01<<6)
 
 struct _grn_obj_format {
   grn_obj columns;
@@ -42,10 +49,15 @@ struct _grn_obj_format {
   int offset;
   int limit;
   int hits_offset;
-  int flags;
+  uint32_t flags;
   grn_obj *expression;
 };
 
+GRN_API grn_rc grn_output_range_normalize(grn_ctx *ctx,
+                                          int size,
+                                          int *offset,
+                                          int *limit);
+
 #define GRN_OBJ_FORMAT_INIT(format,format_nhits,format_offset,format_limit,format_hits_offset) do { \
   GRN_PTR_INIT(&(format)->columns, GRN_OBJ_VECTOR, GRN_ID_NIL);\
   (format)->nhits = (format_nhits);\
@@ -56,25 +68,21 @@ struct _grn_obj_format {
   (format)->expression = NULL;\
 } while (0)
 
-#define GRN_OBJ_FORMAT_FIN(ctx,format) do {\
-  int ncolumns = GRN_BULK_VSIZE(&(format)->columns) / sizeof(grn_obj *);\
-  grn_obj **columns = (grn_obj **)GRN_BULK_HEAD(&(format)->columns);\
-  while (ncolumns--) {\
-    grn_obj *column = *columns;\
-    columns++;\
-    if (grn_obj_is_accessor((ctx), column)) {\
-      grn_obj_close((ctx), column);\
-    }\
-  }\
-  GRN_OBJ_FIN((ctx), &(format)->columns);\
-  if ((format)->expression) { GRN_OBJ_FIN((ctx), (format)->expression); } \
-} while (0)
+/* Deprecated since 10.0.0. Use grn_obj_format_fin() instead. */
+#define GRN_OBJ_FORMAT_FIN(ctx,format) grn_obj_format_fin((ctx), (format))
+GRN_API grn_rc grn_obj_format_fin(grn_ctx *ctx, grn_obj_format *format);
 
 GRN_API void grn_output_obj(grn_ctx *ctx, grn_obj *outbuf, grn_content_type output_type,
                             grn_obj *obj, grn_obj_format *format);
 GRN_API void grn_output_envelope(grn_ctx *ctx, grn_rc rc,
                                  grn_obj *head, grn_obj *body, grn_obj *foot,
                                  const char *file, int line);
+GRN_API void grn_output_envelope_open(grn_ctx *ctx, grn_obj *output);
+GRN_API void grn_output_envelope_close(grn_ctx *ctx,
+                                       grn_obj *output,
+                                       grn_rc rc,
+                                       const char *file,
+                                       int line);
 
 GRN_API void grn_output_array_open(grn_ctx *ctx,
                                    grn_obj *outbuf,
@@ -111,6 +119,10 @@ GRN_API void grn_output_uint64(grn_ctx *ctx,
                                grn_obj *outbuf,
                                grn_content_type output_type,
                                uint64_t value);
+GRN_API void grn_output_float32(grn_ctx *ctx,
+                                grn_obj *outbuf,
+                                grn_content_type output_type,
+                                float value);
 GRN_API void grn_output_float(grn_ctx *ctx,
                               grn_obj *outbuf,
                               grn_content_type output_type,
@@ -141,6 +153,7 @@ GRN_API void grn_ctx_output_int32(grn_ctx *ctx, int32_t value);
 GRN_API void grn_ctx_output_uint32(grn_ctx *ctx, uint32_t value);
 GRN_API void grn_ctx_output_int64(grn_ctx *ctx, int64_t value);
 GRN_API void grn_ctx_output_uint64(grn_ctx *ctx, uint64_t value);
+GRN_API void grn_ctx_output_float32(grn_ctx *ctx, float value);
 GRN_API void grn_ctx_output_float(grn_ctx *ctx, double value);
 GRN_API void grn_ctx_output_cstr(grn_ctx *ctx, const char *value);
 GRN_API void grn_ctx_output_str(grn_ctx *ctx,
@@ -161,6 +174,12 @@ GRN_API void grn_ctx_output_result_set(grn_ctx *ctx,
 GRN_API void grn_ctx_output_table_columns(grn_ctx *ctx,
                                           grn_obj *table,
                                           grn_obj_format *format);
+GRN_API void grn_ctx_output_table_records_open(grn_ctx *ctx,
+                                               int n_records);
+GRN_API void grn_ctx_output_table_records_content(grn_ctx *ctx,
+                                                  grn_obj *table,
+                                                  grn_obj_format *format);
+GRN_API void grn_ctx_output_table_records_close(grn_ctx *ctx);
 GRN_API void grn_ctx_output_table_records(grn_ctx *ctx,
                                           grn_obj *table,
                                           grn_obj_format *format);
