@@ -1,5 +1,5 @@
 /*
-  Copyright(C) 2015 Brazil
+  Copyright(C) 2015-2016 Brazil
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -16,8 +16,7 @@
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
-#ifndef GROONGA_WINDOWS_H
-#define GROONGA_WINDOWS_H
+#pragma once
 
 #ifdef  __cplusplus
 extern "C" {
@@ -30,5 +29,3 @@ GRN_API const char *grn_windows_base_dir(void);
 #ifdef __cplusplus
 }
 #endif
-
-#endif /* GROONGA_WINDOWS_H */
