@@ -1,6 +1,6 @@
 /*
   Copyright(C) 2017  Brazil
-  Copyright(C) 2019-2021  Sutou Kouhei <kou@clear-code.com>
+  Copyright(C) 2019-2022  Sutou Kouhei <kou@clear-code.com>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -201,16 +201,11 @@ namespace grnarrow {
         if (!arrow_type) {
           auto range = object_cache[range_id];
           if (grn_obj_is_table_with_key(ctx, range)) {
-            // TODO: We can't return reference value as dictionary
-            // because arrow::ipc::RecordBatchWriter doesn't support
-            // dictionary delta for now.
-            //
-            // auto domain = object_cache[range->header.domain];
-            // auto dictionary_type =
-            //   grn_column_to_arrow_type(ctx, domain, object_cache);
-            // arrow_type = arrow::dictionary(arrow::int32(), dictionary_type);
             auto domain = object_cache[range->header.domain];
             arrow_type = grn_column_to_arrow_type(ctx, domain, object_cache);
+            if (arrow_type == arrow::utf8()) {
+              arrow_type = arrow::dictionary(arrow::int32(), arrow_type);
+            }
           }
         }
         if (range_flags & GRN_OBJ_VECTOR) {
@@ -228,6 +223,56 @@ namespace grnarrow {
     return nullptr;
   }
 
+  const std::shared_ptr<arrow::DataType> undictionary(
+    const std::shared_ptr<arrow::DataType> data_type)
+  {
+    if (data_type->id() != arrow::Type::DICTIONARY) {
+      return data_type;
+    }
+    auto dictionary_data_type =
+      std::static_pointer_cast<arrow::DictionaryType>(data_type);
+    return dictionary_data_type->value_type();
+  }
+
+  class ArrayBuilderResetFullVisitor : public arrow::TypeVisitor {
+    // We need to execute ResetFull() with an appropriate dictionary length
+    // in order to avoid memory exhaustion and so on.
+    // - Dictionary data in DictionaryBuilder are held in memory until
+    //   ResetFull() is executed.
+    // - Some Apache Arrow implementations (clients) have a 2GB limitation on
+    //   the value size of array: C#.
+  public:
+    ArrayBuilderResetFullVisitor(grn_ctx *ctx,
+                                 arrow::ArrayBuilder *builder)
+      : ctx_(ctx),
+      builder_(builder) {
+    }
+
+    arrow::Status Visit(const arrow::DictionaryType &type) override
+    {
+      const int64_t dictionary_length_threshold = 10000;
+
+      // The value type of Dictionary is always string for now.
+      auto builder = static_cast<arrow::StringDictionaryBuilder *>(builder_);
+      if (builder->dictionary_length() > dictionary_length_threshold) {
+        builder->ResetFull();
+      }
+      return arrow::Status::OK();
+    }
+
+    arrow::Status Visit(const arrow::ListType &type) override
+    {
+      auto builder = static_cast<arrow::ListBuilder *>(builder_);
+      auto value_builder = builder->value_builder();
+      ArrayBuilderResetFullVisitor value_visitor(ctx_, value_builder);
+      return value_builder->type()->Accept(&value_visitor);
+    }
+
+  private:
+    grn_ctx *ctx_;
+    arrow::ArrayBuilder *builder_;
+  };
+
   class RecordAddVisitor : public arrow::ArrayVisitor {
   public:
     RecordAddVisitor(grn_ctx *ctx,
@@ -768,10 +813,12 @@ namespace grnarrow {
     arrow::Status
     load_value(LoadBulk load_bulk) {
       load_bulk();
-      if (grn_obj_cast(ctx_, &buffer_, bulk_, true) != GRN_SUCCESS) {
-        auto ctx = ctx_;
-        auto range = (*object_cache_)[bulk_->header.domain];
-        ERR_CAST(grn_column_, range, &buffer_);
+      if (bulk_->header.domain != GRN_DB_VOID) {
+        if (grn_obj_cast(ctx_, &buffer_, bulk_, true) != GRN_SUCCESS) {
+          auto ctx = ctx_;
+          auto range = (*object_cache_)[bulk_->header.domain];
+          ERR_CAST(grn_column_, range, &buffer_);
+        }
       }
       return arrow::Status::OK();
     }
@@ -789,20 +836,20 @@ namespace grnarrow {
         grn_loader_(grn_loader),
         grn_table_(grn_table),
         record_ids_(record_ids),
+        column_name_(arrow_field->name()),
         grn_column_(nullptr),
         buffer_(),
         object_cache_(object_cache) {
-      const auto &column_name = arrow_field->name();
       if (grn_loader_) {
         grn_column_ = grn_loader_get_column(ctx_,
                                             grn_loader_,
-                                            column_name.data(),
-                                            column_name.size());
+                                            column_name_.data(),
+                                            column_name_.size());
       } else {
         grn_column_ = grn_obj_column(ctx_,
                                      grn_table,
-                                     column_name.data(),
-                                     column_name.size());
+                                     column_name_.data(),
+                                     column_name_.size());
       }
 
       const auto &arrow_type = arrow_field->type();
@@ -816,23 +863,12 @@ namespace grnarrow {
       }
 
       if (!grn_column_) {
-        if (grn_loader_) {
-          char table_name[GRN_TABLE_MAX_KEY_SIZE];
-          auto table_name_size = grn_obj_name(ctx_,
-                                              grn_table_,
-                                              table_name,
-                                              GRN_TABLE_MAX_KEY_SIZE);
-          ERR(GRN_INVALID_ARGUMENT,
-              "[table][load][%.*s] nonexistent column: <%s>",
-              table_name_size, table_name,
-              column_name.data());
-          return;
-        } else {
+        if (!grn_loader_) {
           grn_column_ =
             grn_column_create(ctx_,
                               grn_table_,
-                              column_name.data(),
-                              static_cast<unsigned int>(column_name.size()),
+                              column_name_.data(),
+                              static_cast<unsigned int>(column_name_.size()),
                               NULL,
                               GRN_OBJ_COLUMN_SCALAR,
                               (*object_cache_)[arrow_type_id]);
@@ -841,7 +877,10 @@ namespace grnarrow {
       if (grn_type_id_is_text_family(ctx_, arrow_type_id)) {
         GRN_VALUE_VAR_SIZE_INIT(&buffer_, flags, arrow_type_id);
       } else {
-        grn_id range_id = grn_obj_get_range(ctx, grn_column_);
+        grn_id range_id = GRN_ID_NIL;
+        if (grn_column_) {
+          range_id = grn_obj_get_range(ctx, grn_column_);
+        }
         GRN_VALUE_FIX_SIZE_INIT(&buffer_, flags, range_id);
         if (flags & GRN_OBJ_WITH_WEIGHT) {
           buffer_.header.flags |= GRN_OBJ_WITH_WEIGHT;
@@ -925,6 +964,7 @@ namespace grnarrow {
     grn_loader *grn_loader_;
     grn_obj *grn_table_;
     const grn_id *record_ids_;
+    std::string column_name_;
     grn_obj *grn_column_;
     grn_obj buffer_;
     ObjectCache *object_cache_;
@@ -1009,54 +1049,63 @@ namespace grnarrow {
         if (record_id == GRN_ID_NIL) {
           continue;
         }
+
         GRN_BULK_REWIND(&buffer_);
         ValueLoadVisitor visitor(ctx_, grn_column_, &buffer_, i, object_cache_);
         ARROW_RETURN_NOT_OK(array.Accept(&visitor));
-        if (ctx_->rc == GRN_SUCCESS) {
-          grn_obj_set_value(ctx_, grn_column_, record_id, &buffer_, GRN_OBJ_SET);
+        if (!grn_column_) {
+          auto ctx = ctx_;
+          GRN_DEFINE_NAME(grn_table_);
+          ERR(GRN_INVALID_ARGUMENT,
+              "[table][load][%.*s] nonexistent column: <%s>",
+              name_size, name,
+              column_name_.data());
           if (grn_loader_) {
             grn_loader_add_record_data data;
-            std::string column_name_buffer;
             init_grn_loader_add_record_data(&data,
                                             record_id,
-                                            &buffer_,
-                                            column_name_buffer);
+                                            visitor.original_value());
             grn_loader_on_column_set(ctx_, grn_loader_, &data);
           }
-        } else {
+          continue;
+        }
+
+        if (ctx_->rc != GRN_SUCCESS) {
           if (grn_loader_) {
             grn_loader_add_record_data data;
-            std::string column_name_buffer;
             init_grn_loader_add_record_data(&data,
                                             record_id,
-                                            visitor.original_value(),
-                                            column_name_buffer);
+                                            visitor.original_value());
+            grn_loader_on_column_set(ctx_, grn_loader_, &data);
+          }
+          continue;
+        }
+
+        grn_obj_set_value(ctx_, grn_column_,record_id, &buffer_, GRN_OBJ_SET);
+        if (ctx_->rc != GRN_SUCCESS) {
+          if (grn_loader_) {
+            grn_loader_add_record_data data;
+            init_grn_loader_add_record_data(&data,
+                                            record_id,
+                                            &buffer_);
             grn_loader_on_column_set(ctx_, grn_loader_, &data);
           }
         }
       }
-      return arrow::Status::OK();
+      return check(ctx_, ctx_->rc, "[arrow][column-loader][set-value]");
     }
 
     void init_grn_loader_add_record_data(grn_loader_add_record_data *data,
                                          grn_id record_id,
-                                         grn_obj *value,
-                                         std::string &column_name_buffer)
+                                         grn_obj *value)
     {
       data->table = grn_loader_->table;
       data->depth = 0;
       data->record_value = nullptr;
       data->id = record_id;
       data->key = nullptr;
-      auto ctx = ctx_;
-      GRN_DEFINE_NAME(grn_column_);
-      column_name_buffer.assign(name, name_size);
-      auto delimiter_position = column_name_buffer.find(GRN_DB_DELIMITER);
-      if (delimiter_position != std::string::npos) {
-        column_name_buffer.erase(0, delimiter_position + 1);
-      }
-      data->current.column_name = column_name_buffer.data();
-      data->current.column_name_size = column_name_buffer.size();
+      data->current.column_name = column_name_.data();
+      data->current.column_name_size = column_name_.size();
       data->current.column = grn_column_;
       data->current.value = value;
     }
@@ -1160,6 +1209,9 @@ namespace grnarrow {
         if (!field_type) {
           continue;
         }
+        // We can't use dictionary with FileWriter. It doesn't support
+        // dictionary delta.
+        field_type = undictionary(field_type);
 
         auto field = std::make_shared<arrow::Field>(field_name,
                                                     field_type,
@@ -1168,12 +1220,8 @@ namespace grnarrow {
       };
 
       auto schema = std::make_shared<arrow::Schema>(fields);
-
-#if ARROW_VERSION_MAJOR >= 2
       auto writer_result = arrow::ipc::MakeFileWriter(output, schema);
-#else
-      auto writer_result = arrow::ipc::NewFileWriter(output, schema);
-#endif
+
       if (!check(ctx_,
                  writer_result,
                  "[arrow][dump] failed to create file format writer")) {
@@ -1808,11 +1856,10 @@ namespace grnarrow {
       schema_builder_.Reset();
       schema_ = *schema;
 
-#if ARROW_VERSION_MAJOR >= 2
-      auto writer = arrow::ipc::MakeStreamWriter(&output_, schema_);
-#else
-      auto writer = arrow::ipc::NewStreamWriter(&output_, schema_);
-#endif
+      auto option = arrow::ipc::IpcWriteOptions::Defaults();
+      option.emit_dictionary_deltas = true;
+      auto writer = arrow::ipc::MakeStreamWriter(&output_, schema_, option);
+
       if (!check(ctx_,
                  writer,
                  "[arrow][stream-writer][write-schema] "
@@ -1862,6 +1909,22 @@ namespace grnarrow {
             ">");
     }
 
+    void add_column_int8(int8_t value) {
+      auto column_builder =
+        record_batch_builder_->GetFieldAs<arrow::Int8Builder>(
+          current_column_index_++);
+      auto status = column_builder->Append(value);
+      if (!status.ok()) {
+        return;
+      }
+      std::stringstream context;
+      check(ctx_,
+            status,
+            context <<
+            "[arrow][stream-writer][add-column][int8] " <<
+            "failed to add a column value: <" << value << ">");
+    }
+
     void add_column_int32(int32_t value) {
       auto column_builder =
         record_batch_builder_->GetFieldAs<arrow::Int32Builder>(
@@ -1995,17 +2058,6 @@ namespace grnarrow {
     }
 
     void add_column_record(grn_obj *record) {
-      // TODO: We can't return reference value as dictionary
-      // because arrow::ipc::RecordBatchWriter doesn't support
-      // dictionary delta for now.
-      //
-      // auto column_builder =
-      //   record_batch_builder_->GetFieldAs<arrow::StringDictionaryBuilder>(
-      //     current_column_index_++);
-
-      auto column_builder =
-        record_batch_builder_->GetFieldAs<arrow::StringBuilder>(
-          current_column_index_++);
       auto table = object_cache_[record->header.domain];
       char key[GRN_TABLE_MAX_KEY_SIZE];
       auto key_size = grn_table_get_key(ctx_,
@@ -2013,6 +2065,16 @@ namespace grnarrow {
                                         GRN_RECORD_VALUE(record),
                                         key,
                                         sizeof(key));
+      switch (table->header.domain) {
+      case GRN_DB_INT32 :
+        add_column_int32(*reinterpret_cast<int32_t *>(key));
+        return;
+      default :
+        break;
+      }
+      auto column_builder =
+        record_batch_builder_->GetFieldAs<arrow::StringDictionaryBuilder>(
+          current_column_index_++);
       auto status =
         column_builder->Append(arrow::util::string_view(key, key_size));
       if (status.ok()) {
@@ -2046,8 +2108,8 @@ namespace grnarrow {
         auto raw_value_builder = column_builder->value_builder();
         if (grn_obj_is_table_with_key(ctx_, domain)) {
           auto value_builder =
-            static_cast<arrow::StringBuilder *>(raw_value_builder);
-          for (long i = 0; i < n; ++i) {
+            static_cast<arrow::StringDictionaryBuilder *>(raw_value_builder);
+          for (size_t i = 0; i < n; ++i) {
             auto record_id =
               *reinterpret_cast<grn_id *>(raw_elements + (element_size * i));
             char key[GRN_TABLE_MAX_KEY_SIZE];
@@ -2065,7 +2127,7 @@ namespace grnarrow {
         } else {
           auto value_builder =
             static_cast<arrow::Int32Builder *>(raw_value_builder);
-          for (long i = 0; i < n; ++i) {
+          for (size_t i = 0; i < n; ++i) {
             // TODO: check type
             auto element =
               *reinterpret_cast<int32_t *>(raw_elements + (element_size * i));
@@ -2110,6 +2172,14 @@ namespace grnarrow {
               "[arrow][stream-writer][flush] "
               "failed to write flushed record batch");
       }
+
+      auto n_fields = record_batch_builder_->num_fields();
+      for (int i = 0; i < n_fields; ++i) {
+        auto builder = record_batch_builder_->GetField(i);
+        auto visitor = ArrayBuilderResetFullVisitor(ctx_, builder);
+        builder->type()->Accept(&visitor);
+      }
+
       n_records_ = 0;
     }
 
@@ -2206,7 +2276,7 @@ namespace grn {
 
       ::arrow::Status add_column(grn_obj *column,
                                  grn_table_cursor *cursor) {
-        const auto arrow_type =
+        auto arrow_type =
           grn_column_to_arrow_type(ctx_, column, object_cache_);
         if (!arrow_type) {
           grn::TextBulk inspected(ctx_);
@@ -2215,6 +2285,7 @@ namespace grn {
             "[arrow][array-builder][add-column] "
             "unsupported column: ", inspected.value());
         }
+        arrow_type = grnarrow::undictionary(arrow_type);
         if (!builder_) {
           ARROW_RETURN_NOT_OK(
             ::arrow::MakeBuilder(::arrow::default_memory_pool(),
@@ -2472,7 +2543,8 @@ grn_arrow_stream_loader_open(grn_ctx *ctx,
   GRN_API_ENTER;
 #ifdef GRN_WITH_APACHE_ARROW
   grn_arrow_stream_loader *arrow_stream_loader;
-  arrow_stream_loader = GRN_MALLOCN(grn_arrow_stream_loader, 1);
+  arrow_stream_loader = static_cast<grn_arrow_stream_loader *>(
+    GRN_CALLOC(sizeof(grn_arrow_stream_loader)));
   arrow_stream_loader->loader = new grnarrow::StreamLoader(ctx, loader);
   GRN_API_RETURN(arrow_stream_loader);
 #else
@@ -2530,7 +2602,8 @@ grn_arrow_stream_writer_open(grn_ctx *ctx,
   GRN_API_ENTER;
 #ifdef GRN_WITH_APACHE_ARROW
   grn_arrow_stream_writer *arrow_stream_writer;
-  arrow_stream_writer = GRN_MALLOCN(grn_arrow_stream_writer, 1);
+  arrow_stream_writer = static_cast<grn_arrow_stream_writer *>(
+    GRN_CALLOC(sizeof(grn_arrow_stream_writer)));
   arrow_stream_writer->writer = new grnarrow::StreamWriter(ctx, output_buffer);
   GRN_API_RETURN(arrow_stream_writer);
 #else
@@ -2650,6 +2723,22 @@ grn_arrow_stream_writer_add_column_string(grn_ctx *ctx,
   GRN_API_RETURN(ctx->rc);
 }
 
+grn_rc
+grn_arrow_stream_writer_add_column_int8(grn_ctx *ctx,
+                                        grn_arrow_stream_writer *writer,
+                                        int8_t value)
+{
+  GRN_API_ENTER;
+#ifdef GRN_WITH_APACHE_ARROW
+  writer->writer->add_column_int8(value);
+#else
+  ERR(GRN_FUNCTION_NOT_IMPLEMENTED,
+      "[arrow][stream-writer][add-column][int8] "
+      "Apache Arrow support isn't enabled");
+#endif
+  GRN_API_RETURN(ctx->rc);
+}
+
 grn_rc
 grn_arrow_stream_writer_add_column_int32(grn_ctx *ctx,
                                          grn_arrow_stream_writer *writer,
