@@ -1,6 +1,6 @@
-/* -*- c-basic-offset: 2 -*- */
 /*
-  Copyright(C) 2009-2017 Brazil
+  Copyright(C) 2009-2017  Brazil
+  Copyright(C) 2021  Sutou Kouhei <kou@clear-code.com>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -61,6 +61,8 @@ typedef union _grn_cache_entry_persistent {
 struct _grn_cache {
   union {
     struct {
+      /* next and prev is the same layout as grn_cache_entry_memory to
+       * use this struct as the root entry. */
       grn_cache_entry_memory *next;
       grn_cache_entry_memory *prev;
       grn_hash *hash;
@@ -118,8 +120,8 @@ grn_get_default_cache_base_path(void)
 static void
 grn_cache_open_memory(grn_ctx *ctx, grn_cache *cache)
 {
-  cache->impl.memory.next = (grn_cache_entry_memory *)cache;
-  cache->impl.memory.prev = (grn_cache_entry_memory *)cache;
+  cache->impl.memory.next = (grn_cache_entry_memory *)&(cache->impl.memory);
+  cache->impl.memory.prev = (grn_cache_entry_memory *)&(cache->impl.memory);
   cache->impl.memory.hash = grn_hash_create(cache->ctx,
                                             NULL,
                                             GRN_CACHE_MAX_KEY_SIZE,
@@ -366,11 +368,12 @@ grn_persistent_cache_open(grn_ctx *ctx, const char *base_path)
 static void
 grn_cache_close_memory(grn_ctx *ctx, grn_cache *cache)
 {
-  grn_cache_entry_memory *vp;
-
-  GRN_HASH_EACH(ctx, cache->impl.memory.hash, id, NULL, NULL, &vp, {
-    grn_obj_close(ctx, vp->value);
-  });
+  GRN_HASH_EACH_BEGIN(ctx, cache->impl.memory.hash, cursor, id) {
+    void *value;
+    grn_hash_cursor_get_value(ctx, cursor, &value);
+    grn_cache_entry_memory *entry = value;
+    grn_obj_close(ctx, entry->value);
+  } GRN_HASH_EACH_END(ctx, cursor);
   grn_hash_close(ctx, cache->impl.memory.hash);
   MUTEX_FIN(cache->impl.memory.mutex);
 }
@@ -528,6 +531,9 @@ grn_cache_expire_memory_without_lock(grn_cache *cache, int32_t size)
   while (ce0 != ce0->prev && size--) {
     grn_cache_expire_entry_memory(cache, ce0->prev);
   }
+  if (size == 0) {
+    grn_hash_truncate(cache->ctx, cache->impl.memory.hash);
+  }
 }
 
 static void
@@ -599,7 +605,7 @@ grn_cache_set_max_n_entries_persistent(grn_ctx *ctx,
   if (n < current_max_n_entries) {
     grn_cache_expire_persistent_without_lock(cache, current_max_n_entries - n);
   }
-  grn_io_unlock(keys->io);
+  grn_io_unlock(ctx, keys->io);
 
   return GRN_SUCCESS;
 }
@@ -643,7 +649,7 @@ grn_cache_get_max_n_entries_persistent(grn_ctx *ctx, grn_cache *cache)
                                                         GRN_CACHE_PERSISTENT_METADATA_ID,
                                                         NULL);
   current_max_n_entries = metadata_entry->metadata.max_nentries;
-  grn_io_unlock(keys->io);
+  grn_io_unlock(ctx, keys->io);
 
   return current_max_n_entries;
 }
@@ -698,7 +704,7 @@ grn_cache_get_statistics_persistent(grn_ctx *ctx, grn_cache *cache,
   statistics->nfetches = metadata_entry->metadata.nfetches;
   statistics->nhits = metadata_entry->metadata.nhits;
 
-  grn_io_unlock(keys->io);
+  grn_io_unlock(ctx, keys->io);
 }
 
 void
@@ -818,7 +824,7 @@ grn_cache_fetch_persistent(grn_ctx *ctx, grn_cache *cache,
   }
 
 exit :
-  grn_io_unlock(keys->io);
+  grn_io_unlock(ctx, keys->io);
 
   return rc;
 }
@@ -970,7 +976,7 @@ grn_cache_update_persistent(grn_ctx *ctx, grn_cache *cache,
   }
 
 exit :
-  grn_io_unlock(keys->io);
+  grn_io_unlock(ctx, keys->io);
 }
 
 void
@@ -1008,7 +1014,7 @@ grn_cache_expire_persistent(grn_cache *cache, int32_t size)
 
   grn_cache_expire_persistent_without_lock(cache, size);
 
-  grn_io_unlock(keys->io);
+  grn_io_unlock(ctx, keys->io);
 }
 
 void
