@@ -1,7 +1,6 @@
-/* -*- c-basic-offset: 2 -*- */
 /*
   Copyright(C) 2009-2017  Brazil
-  Copyright(C) 2018-2020  Sutou Kouhei <kou@clear-code.com>
+  Copyright(C) 2018-2021  Sutou Kouhei <kou@clear-code.com>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -21,6 +20,7 @@
 #include "grn_column.h"
 #include "grn_db.h"
 #include "grn_ii.h"
+#include "grn_util.h"
 
 grn_column_flags
 grn_column_get_flags(grn_ctx *ctx, grn_obj *column)
@@ -125,7 +125,7 @@ grn_column_cache_open(grn_ctx *ctx, grn_obj *column)
     GRN_API_RETURN(NULL);
   }
 
-  cache = GRN_MALLOC(sizeof(grn_column_cache));
+  cache = GRN_CALLOC(sizeof(grn_column_cache));
   if (!cache) {
     ERR(GRN_NO_MEMORY_AVAILABLE,
         "[column-cache][open] failed to allocate memory");
@@ -148,7 +148,7 @@ grn_column_cache_close(grn_ctx *ctx, grn_column_cache *cache)
     GRN_API_RETURN();
   }
 
-  GRN_RA_CACHE_FIN(cache->ra, &(cache->ra_cache));
+  GRN_RA_CACHE_FIN(ctx, cache->ra, &(cache->ra_cache));
   GRN_FREE(cache);
 
   GRN_API_RETURN();
@@ -189,32 +189,150 @@ grn_column_cache_ref(grn_ctx *ctx,
   GRN_API_RETURN(value);
 }
 
+static void
+grn_column_copy_same_table_report_invalid_reference_value(grn_ctx *ctx,
+                                                          grn_obj *table,
+                                                          grn_obj *from_column,
+                                                          grn_id id,
+                                                          grn_id value)
+{
+  GRN_DEFINE_NAME(from_column);
+  char key[GRN_TABLE_MAX_KEY_SIZE];
+  int key_size = 0;
+  if (grn_obj_is_table_with_key(ctx, table)) {
+    key_size = grn_table_get_key(ctx, table, id, key, sizeof(key));
+  }
+  grn_obj inspected_key;
+  GRN_TEXT_INIT(&inspected_key, 0);
+  grn_inspect_key(ctx, &inspected_key, table, key, key_size);
+  GRN_LOG(ctx,
+          GRN_LOG_WARNING,
+          "[column][copy][%.*s] "
+          "ignore invalid reference value: "
+          "value:%u, "
+          "id:%u, "
+          "key:%.*s",
+          name_size, name,
+          value,
+          id,
+          (int)GRN_TEXT_LEN(&inspected_key),
+          GRN_TEXT_VALUE(&inspected_key));
+  GRN_OBJ_FIN(ctx, &inspected_key);
+}
+
 static void
 grn_column_copy_same_table(grn_ctx *ctx,
                            grn_obj *table,
                            grn_obj *from_column,
                            grn_obj *to_column)
 {
-  grn_table_cursor *cursor;
-  grn_id id;
-  grn_obj value;
-
-  cursor = grn_table_cursor_open(ctx, table,
-                                 NULL, 0,
-                                 NULL, 0,
-                                 0, -1, 0);
-  if (!cursor) {
-    return;
+  grn_id from_range_id = GRN_ID_NIL;
+  grn_obj_flags from_range_flags = 0;
+  grn_id to_range_id = GRN_ID_NIL;
+  grn_obj_flags to_range_flags = 0;
+  bool need_validate = false;
+  if (grn_obj_is_reference_column(ctx, from_column) &&
+      grn_obj_is_reference_column(ctx, to_column)) {
+    grn_obj_get_range_info(ctx, from_column, &from_range_id, &from_range_flags);
+    grn_obj_get_range_info(ctx, to_column, &to_range_id, &to_range_flags);
+    need_validate = (from_range_id == to_range_id &&
+                     from_range_flags == to_range_flags);
   }
 
+  grn_obj value;
   GRN_VOID_INIT(&value);
-  while ((id = grn_table_cursor_next(ctx, cursor)) != GRN_ID_NIL) {
-    GRN_BULK_REWIND(&value);
-    grn_obj_get_value(ctx, from_column, id, &value);
-    grn_obj_set_value(ctx, to_column, id, &value, GRN_OBJ_SET);
+  if (need_validate) {
+    grn_obj validated_value;
+    GRN_RECORD_INIT(&validated_value, to_range_flags, to_range_id);
+    bool is_uvector = grn_obj_is_uvector(ctx, &validated_value);
+    grn_obj *range = grn_ctx_at(ctx, to_range_id);
+    GRN_TABLE_EACH_BEGIN(ctx, table, cursor, id) {
+      GRN_BULK_REWIND(&value);
+      grn_obj_get_value(ctx, from_column, id, &value);
+      GRN_BULK_REWIND(&validated_value);
+      if (is_uvector) {
+        uint32_t n_elements = grn_uvector_size(ctx, &value);
+        uint32_t i;
+        for (i = 0; i < n_elements; i++) {
+          float weight;
+          grn_id record_id = grn_uvector_get_element_record(ctx,
+                                                            &value,
+                                                            i,
+                                                            &weight);
+          if (record_id == GRN_ID_NIL ||
+              record_id >= GRN_ID_MAX ||
+              !grn_table_at(ctx, range, record_id)) {
+            grn_column_copy_same_table_report_invalid_reference_value(
+              ctx,
+              table,
+              from_column,
+              id,
+              record_id);
+            continue;
+          }
+          grn_uvector_add_element_record(ctx,
+                                         &validated_value,
+                                         record_id,
+                                         weight);
+        }
+      } else {
+        if (GRN_BULK_VSIZE(&value) > 0) {
+          grn_id record_id = GRN_RECORD_VALUE(&value);
+          if (record_id == GRN_ID_NIL ||
+              record_id >= GRN_ID_MAX ||
+              !grn_table_at(ctx, range, record_id)) {
+            grn_column_copy_same_table_report_invalid_reference_value(
+              ctx,
+              table,
+              from_column,
+              id,
+              record_id);
+          } else {
+            GRN_RECORD_SET(ctx, &validated_value, record_id);
+          }
+        }
+      }
+      grn_obj_set_value(ctx, to_column, id, &validated_value, GRN_OBJ_SET);
+    } GRN_TABLE_EACH_END(ctx, cursor);
+    grn_obj_unref(ctx, range);
+    GRN_OBJ_FIN(ctx, &validated_value);
+  } else {
+    GRN_TABLE_EACH_BEGIN(ctx, table, cursor, id) {
+      GRN_BULK_REWIND(&value);
+      grn_obj_get_value(ctx, from_column, id, &value);
+      grn_obj_set_value(ctx, to_column, id, &value, GRN_OBJ_SET);
+    } GRN_TABLE_EACH_END(ctx, cursor);
   }
   GRN_OBJ_FIN(ctx, &value);
-  grn_table_cursor_close(ctx, cursor);
+}
+
+static void
+grn_column_copy_array(grn_ctx *ctx,
+                      grn_obj *from_table,
+                      grn_obj *from_column,
+                      grn_obj *to_table,
+                      grn_obj *to_column)
+{
+  grn_obj value;
+  GRN_VOID_INIT(&value);
+  GRN_TABLE_EACH_BEGIN(ctx, from_table, cursor, from_id) {
+    grn_id to_id = grn_table_at(ctx, to_table, from_id);
+    if (to_id != from_id) {
+      GRN_DEFINE_NAME_CUSTOM(from_table, from_name);
+      GRN_DEFINE_NAME_CUSTOM(to_table, to_name);
+      ERR(GRN_INVALID_ARGUMENT,
+          "[column][copy] target ID doesn't exist: <%u>: "
+          "<%.*s> -> <%.*s>",
+          from_id,
+          from_name_size, from_name,
+          to_name_size, to_name);
+      break;
+    }
+    GRN_BULK_REWIND(&value);
+    grn_obj_get_value(ctx, from_column, from_id, &value);
+    grn_obj_set_value(ctx, to_column, to_id, &value, GRN_OBJ_SET);
+  } GRN_TABLE_EACH_END(ctx, cursor);
+  GRN_OBJ_FIN(ctx, &value);
 }
 
 static void
@@ -397,7 +515,7 @@ grn_column_copy(grn_ctx *ctx, grn_obj *from, grn_obj *to)
   to_table = grn_ctx_at(ctx, to->header.domain);
   if ((from_table->header.type == GRN_TABLE_NO_KEY ||
        to_table->header.type == GRN_TABLE_NO_KEY) &&
-      from_table != to_table) {
+      (from_table->header.type != to_table->header.type)) {
     char from_name[GRN_TABLE_MAX_KEY_SIZE];
     int from_name_size;
     char to_name[GRN_TABLE_MAX_KEY_SIZE];
@@ -415,6 +533,10 @@ grn_column_copy(grn_ctx *ctx, grn_obj *from, grn_obj *to)
 
   if (from_table == to_table) {
     grn_column_copy_same_table(ctx, from_table, from, to);
+  } else if (from_table->header.type == GRN_TABLE_NO_KEY) {
+    grn_column_copy_array(ctx,
+                          from_table, from,
+                          to_table, to);
   } else if (from_table->header.domain == to_table->header.domain) {
     grn_column_copy_same_key_type(ctx,
                                   from_table, from,
@@ -428,4 +550,3 @@ grn_column_copy(grn_ctx *ctx, grn_obj *from, grn_obj *to)
 exit :
   GRN_API_RETURN(ctx->rc);
 }
-
