@@ -1,6 +1,6 @@
 /*
-  Copyright(C) 2009-2018 Brazil
-  Copyright(C) 2018-2019 Sutou Kouhei <kou@clear-code.com>
+  Copyright(C) 2009-2018  Brazil
+  Copyright(C) 2018-2021  Sutou Kouhei <kou@clear-code.com>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -127,7 +127,7 @@ grn_msg_open(grn_ctx *ctx, grn_com *com, grn_com_queue *old)
       return NULL;
     }
     GRN_BULK_REWIND(&msg->qe.obj);
-  } else if ((msg = GRN_MALLOCN(grn_msg, 1))) {
+  } else if ((msg = GRN_CALLOC(sizeof(grn_msg)))) {
     GRN_OBJ_INIT(&msg->qe.obj, GRN_MSG, 0, GRN_DB_TEXT);
     msg->qe.obj.header.impl_flags |= GRN_OBJ_ALLOCATED;
     msg->ctx = ctx;
@@ -350,7 +350,7 @@ grn_com_event_add(grn_ctx *ctx, grn_com_event *ev, grn_sock fd, int events, grn_
 {
   grn_com *c;
   /* todo : expand events */
-  if (!ev || *ev->hash->n_entries == ev->max_nevents) {
+  if (!ev || *ev->hash->n_entries == (uint32_t)(ev->max_nevents)) {
     if (ev) { GRN_LOG(ctx, GRN_LOG_ERROR, "too many connections (%d)", ev->max_nevents); }
     return GRN_INVALID_ARGUMENT;
   }
@@ -710,7 +710,7 @@ grn_com_send_http(grn_ctx *ctx, grn_com *cs, const char *path, uint32_t path_len
   if ((ret = send(cs->fd, GRN_BULK_HEAD(&buf), GRN_BULK_VSIZE(&buf), MSG_NOSIGNAL|flags)) == -1) {
     SOERR("send");
   }
-  if (ret != GRN_BULK_VSIZE(&buf)) {
+  if ((size_t)ret != GRN_BULK_VSIZE(&buf)) {
     GRN_LOG(ctx, GRN_LOG_NOTICE, "send %d != %d", (int)ret, (int)GRN_BULK_VSIZE(&buf));
   }
   grn_obj_close(ctx, &buf);
@@ -762,7 +762,7 @@ grn_com_send(grn_ctx *ctx, grn_com *cs,
       rc = ctx->rc;
     }
   }
-  if (ret != whole_size) {
+  if ((size_t)ret != whole_size) {
     GRN_LOG(ctx, GRN_LOG_ERROR,
             "sendmsg(%" GRN_FMT_SOCKET "): %" GRN_FMT_LLD " < %" GRN_FMT_LLU,
             cs->fd, (long long int)ret, (unsigned long long int)whole_size);
@@ -1136,7 +1136,7 @@ grn_com_sopen(grn_ctx *ctx, grn_com_event *ev,
     cs->opaque = NULL;
     GRN_COM_QUEUE_INIT(&cs->new_);
   } else {
-    if (!(cs = GRN_MALLOC(sizeof(grn_com)))) { goto exit; }
+    if (!(cs = GRN_CALLOC(sizeof(grn_com)))) { goto exit; }
     cs->fd = lfd;
   }
   cs->accepting = GRN_TRUE;
@@ -1172,7 +1172,7 @@ grn_edges_add(grn_ctx *ctx, grn_com_addr *addr, int *added)
     grn_edge *edge;
     grn_id id = grn_hash_add(ctx, grn_edges, addr, sizeof(grn_com_addr),
                              (void **)&edge, added);
-    grn_io_unlock(grn_edges->io);
+    grn_io_unlock(ctx, grn_edges->io);
     if (id) { edge->id = id; }
     return edge;
   }
@@ -1183,7 +1183,7 @@ grn_edges_delete(grn_ctx *ctx, grn_edge *edge)
 {
   if (!grn_io_lock(ctx, grn_edges->io, grn_lock_timeout)) {
     grn_hash_delete_by_id(ctx, grn_edges, edge->id, NULL);
-    grn_io_unlock(grn_edges->io);
+    grn_io_unlock(ctx, grn_edges->io);
   }
 }
 
