@@ -1,5 +1,5 @@
-/* -*- c-basic-offset: 2 -*- */
-/* Copyright(C) 2011 Brazil
+/*
+  Copyright(C) 2011-2016 Brazil
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -15,15 +15,14 @@
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
-#ifndef GRN_DAT_KEY_HPP_
-#define GRN_DAT_KEY_HPP_
+#pragma once
 
 #include "string.hpp"
 
 namespace grn {
 namespace dat {
 
-class Key {
+class GRN_DAT_API Key {
  public:
   const UInt8 &operator[](UInt32 i) const {
     GRN_DAT_DEBUG_THROW_IF(i >= length());
@@ -93,12 +92,12 @@ class Key {
   }
 
  private:
-  const UInt32 id_and_length_low_;
-  const UInt8 length_high_;
-  const UInt8 buf_[3];
+  UInt32 id_and_length_low_;
+  UInt8 length_high_;
+  UInt8 buf_[3];
 
   // Disallows instantiation.
-  Key() : id_and_length_low_(INVALID_KEY_ID << 4), length_high_(0), buf_() {}
+  Key() : id_and_length_low_(INVALID_KEY_ID << 4), length_high_(0) {}
   ~Key() {}
 
   // Disallows copy and assignment.
@@ -108,5 +107,3 @@ class Key {
 
 }  // namespace dat
 }  // namespace grn
-
-#endif  // GRN_DAT_KEY_HPP_
