@@ -1,5 +1,5 @@
-/* -*- c-basic-offset: 2 -*- */
-/* Copyright(C) 2011 Brazil
+/*
+  Copyright(C) 2011-2016 Brazil
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -12,11 +12,10 @@
 
   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free Software
-  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
-#ifndef GRN_DAT_NODE_HPP_
-#define GRN_DAT_NODE_HPP_
+#pragma once
 
 // See base.hpp and check.hpp for details.
 #include "base.hpp"
@@ -25,7 +24,7 @@
 namespace grn {
 namespace dat {
 
-class Node {
+class GRN_DAT_API Node {
  public:
   Node() : base_(), check_() {}
 
@@ -125,5 +124,3 @@ class Node {
 
 }  // namespace dat
 }  // namespace grn
-
-#endif  // GRN_DAT_NODE_HPP_
