@@ -1,7 +1,6 @@
-/* -*- c-basic-offset: 2 -*- */
 /*
-  Copyright(C) 2009-2016 Brazil
-  Copyright(C) 2020 Sutou Kouhei <kou@clear-code.com>
+  Copyright(C) 2009-2016  Brazil
+  Copyright(C) 2020-2021  Sutou Kouhei <kou@clear-code.com>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -29,7 +28,6 @@
 #define GRN_BREAK_POINT raise(SIGTRAP)
 #endif /* HAVE_SIGNAL_H */
 
-#include "grn_io.h"
 #include "grn_alloc.h"
 #include "grn_time.h"
 
@@ -90,6 +88,9 @@ void grn_log_reopen(grn_ctx *ctx);
 GRN_API grn_rc grn_ctx_sendv(grn_ctx *ctx, int argc, char **argv, int flags);
 void grn_ctx_set_keep_command(grn_ctx *ctx, grn_obj *command);
 
+grn_ctx *grn_ctx_pull_child(grn_ctx *ctx);
+grn_rc grn_ctx_release_child(grn_ctx *ctx, grn_ctx *child_ctx);
+
 grn_content_type grn_get_ctype(grn_obj *var);
 grn_content_type grn_content_type_parse(grn_ctx *ctx,
                                         grn_obj *var,
