@@ -1,4 +1,3 @@
-/* -*- c-basic-offset: 2 -*- */
 /*
   Copyright(C) 2009-2018  Brazil
   Copyright(C) 2018-2021  Sutou Kouhei <kou@clear-code.com>
@@ -192,8 +191,20 @@ struct _grn_ctx_impl {
   } temporary_open_spaces;
 
   grn_hash *variables;
+
+  struct {
+    grn_wal_role role;
+  } wal;
+
+  struct {
+    grn_critical_section lock;
+    grn_obj pool;
+  } children;
+  grn_ctx *parent;
 };
 
+#define GRN_CTX_GET_WAL_ROLE(ctx) ((ctx)->impl->wal.role)
+
 void grn_ctx_impl_columns_cache_delete(grn_ctx *ctx, grn_id table_id);
 void grn_ctx_impl_columns_cache_clear(grn_ctx *ctx);
 
