@@ -1,4 +1,3 @@
-/* -*- c-basic-offset: 2 -*- */
 /*
   Copyright(C) 2013-2016  Brazil
   Copyright(C) 2020  Sutou Kouhei <kou@clear-code.com>
@@ -45,6 +44,15 @@ extern "C" {
   grn_gctx.rc = GRN_SUCCESS;\
 } while (0)
 
+static grn_inline grn_rc
+grn_ctx_rc_propagate(grn_ctx *ctx, grn_rc rc)
+{
+  if (rc != GRN_SUCCESS) {
+    return rc;
+  }
+  return ctx->rc;
+}
+
 GRN_API grn_bool grn_ctx_impl_should_log(grn_ctx *ctx);
 GRN_API void grn_ctx_impl_set_current_error_message(grn_ctx *ctx);
 
