@@ -129,7 +129,7 @@ void grn_scan_info_set_quorum_threshold(scan_info *si, int quorum_threshold);
 grn_bool grn_scan_info_push_arg(grn_ctx *ctx, scan_info *si, grn_obj *arg);
 grn_obj *grn_scan_info_get_arg(grn_ctx *ctx, scan_info *si, int i);
 int grn_scan_info_get_start_position(scan_info *si);
-void grn_scan_info_set_start_position(scan_info *si, int start);
+void grn_scan_info_set_start_position(scan_info *si, uint32_t start);
 void grn_scan_info_reset_position(scan_info *si);
 
 float grn_expr_code_get_weight(grn_ctx *ctx, grn_expr_code *ec, uint32_t *offset);
