@@ -166,6 +166,7 @@ struct _grn_array {
   grn_io *io;
   struct grn_array_header *header;
   uint32_t *lock;
+  bool wal_touched;
 
   /* For grn_tiny_array. */
   uint32_t n_garbages_buf;
@@ -197,6 +198,7 @@ uint32_t grn_array_get_flags(grn_ctx *ctx, grn_array *array);
 grn_rc grn_array_truncate(grn_ctx *ctx, grn_array *array);
 grn_rc grn_array_copy_sort_key(grn_ctx *ctx, grn_array *array,
                                grn_table_sort_key *keys, int n_keys);
+grn_rc grn_array_wal_recover(grn_ctx *ctx, grn_array *array);
 grn_rc grn_array_warm(grn_ctx *ctx, grn_array *array);
 
 /**** grn_hash ****/
@@ -211,6 +213,8 @@ typedef struct _grn_hash_header_common grn_hash_header_common;
 typedef struct _grn_hash_header_normal grn_hash_header_normal;
 typedef struct _grn_hash_header_large  grn_hash_header_large;
 
+typedef struct _grn_hash_wal_add_entry_data grn_hash_wal_add_entry_data;
+
 struct _grn_hash {
   grn_db_obj obj;
   grn_ctx *ctx;
@@ -256,6 +260,8 @@ struct _grn_hash {
   grn_id garbages;
   grn_tiny_array a;
   grn_tiny_bitmap bitmap;
+
+  grn_hash_wal_add_entry_data *wal_data;
 };
 
 #define GRN_HASH_HEADER_COMMON_FIELDS\
@@ -275,7 +281,8 @@ struct _grn_hash {
   grn_id normalizer;\
   uint32_t truncated;\
   uint64_t curr_key_large;\
-  uint32_t reserved[12]
+  uint64_t wal_id;\
+  uint32_t reserved[10]
 
 struct _grn_hash_header_common {
   GRN_HASH_HEADER_COMMON_FIELDS;
@@ -330,6 +337,8 @@ struct _grn_table_sort_optarg {
   int offset;
 };
 
+void grn_hash_init_from_env(void);
+
 GRN_API int grn_hash_sort(grn_ctx *ctx, grn_hash *hash, int limit,
                           grn_array *result, grn_table_sort_optarg *optarg);
 
@@ -406,6 +415,7 @@ grn_bool grn_hash_is_large_total_key_size(grn_ctx *ctx, grn_hash *hash);
 uint64_t grn_hash_total_key_size(grn_ctx *ctx, grn_hash *hash);
 uint64_t grn_hash_max_total_key_size(grn_ctx *ctx, grn_hash *hash);
 
+grn_rc grn_hash_wal_recover(grn_ctx *ctx, grn_hash *hash);
 grn_rc grn_hash_warm(grn_ctx *ctx, grn_hash *hash);
 
 #ifdef __cplusplus
