@@ -1,4 +1,3 @@
-/* -*- c-basic-offset: 2 -*- */
 /*
   Copyright(C) 2009-2018  Brazil
   Copyright(C) 2019-2021  Sutou Kouhei <kou@clear-code.com>
@@ -49,6 +48,8 @@ struct _grn_ii {
     grn_ii_header_normal *normal;
     grn_ii_header_large *large;
   } header;
+
+  bool wal_touched;
 };
 
 /* BGQ is buffer garbage queue? */
@@ -184,6 +185,8 @@ struct _grn_select_optarg {
   int similarity_threshold;
   int max_interval;
   int *weight_vector;
+  float *weight_vector_float;
+  float weight_float;
   int vector_size;
   int (*func)(grn_ctx *, grn_hash *, const void *, int, void *);
   void *func_arg;
@@ -195,6 +198,7 @@ struct _grn_select_optarg {
   grn_match_info *match_info;
   int quorum_threshold;
   int additional_last_interval;
+  grn_obj *query_options;
 };
 
 GRN_API grn_rc grn_ii_column_update(grn_ctx *ctx, grn_ii *ii, grn_id id,
@@ -225,6 +229,9 @@ typedef struct grn_ii_builder_options grn_ii_builder_options;
 grn_rc grn_ii_build2(grn_ctx *ctx, grn_ii *ii,
                      const grn_ii_builder_options *options);
 
+grn_rc grn_ii_wal_recover(grn_ctx *ctx, grn_ii *ii);
+grn_rc grn_ii_warm(grn_ctx *ctx, grn_ii *ii);
+
 #ifdef __cplusplus
 }
 #endif
