@@ -1,6 +1,6 @@
 /*
   Copyright(C) 2009-2018  Brazil
-  Copyright(C) 2019-2021  Sutou Kouhei <kou@clear-code.com>
+  Copyright(C) 2019-2022  Sutou Kouhei <kou@clear-code.com>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -57,6 +57,9 @@ void grn_proc_init_index_column_diff(grn_ctx *ctx);
 void grn_proc_init_lock_acquire(grn_ctx *ctx);
 void grn_proc_init_lock_clear(grn_ctx *ctx);
 void grn_proc_init_lock_release(grn_ctx *ctx);
+void grn_proc_init_log_level(grn_ctx *ctx);
+void grn_proc_init_log_put(grn_ctx *ctx);
+void grn_proc_init_log_reopen(grn_ctx *ctx);
 void grn_proc_init_normalize(grn_ctx *ctx);
 void grn_proc_init_object_exist(grn_ctx *ctx);
 void grn_proc_init_object_inspect(grn_ctx *ctx);
@@ -84,6 +87,8 @@ void grn_proc_init_table_list(grn_ctx *ctx);
 void grn_proc_init_table_remove(grn_ctx *ctx);
 void grn_proc_init_table_rename(grn_ctx *ctx);
 void grn_proc_init_table_tokenize(grn_ctx *ctx);
+void grn_proc_init_thread_dump(grn_ctx *ctx);
+void grn_proc_init_thread_limit(grn_ctx *ctx);
 void grn_proc_init_tokenize(grn_ctx *ctx);
 
 grn_bool grn_proc_option_value_bool(grn_ctx *ctx,
@@ -102,6 +107,14 @@ bool grn_proc_get_value_bool(grn_ctx *ctx,
                              grn_obj *value,
                              bool default_value,
                              const char *tag);
+int32_t grn_proc_get_value_int32(grn_ctx *ctx,
+                                 grn_obj *value,
+                                 int32_t default_value_raw,
+                                 const char *tag);
+uint32_t grn_proc_get_value_uint32(grn_ctx *ctx,
+                                   grn_obj *value,
+                                   uint32_t default_value_raw,
+                                   const char *tag);
 int64_t grn_proc_get_value_int64(grn_ctx *ctx,
                                  grn_obj *value,
                                  int64_t default_value_raw,
