@@ -1,6 +1,5 @@
-/* -*- c-basic-offset: 2 -*- */
 /*
-  Copyright(C) 2015 Brazil
+  Copyright(C) 2015-2016 Brazil
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -15,8 +14,8 @@
   License along with this library; if not, write to the Free Software
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
-#ifndef GRN_SCORER_H
-#define GRN_SCORER_H
+
+#pragma once
 
 #include "grn_ctx.h"
 #include "grn_db.h"
@@ -47,5 +46,3 @@ struct _grn_scorer_matched_record {
 #ifdef __cplusplus
 }
 #endif
-
-#endif /* GRN_SCORER_H */
