@@ -43,7 +43,8 @@ struct grn_ra_header {
   uint32_t element_size;
   uint32_t nrecords; /* nrecords is not maintained by default */
   grn_column_flags flags;
-  uint32_t reserved[9];
+  uint64_t wal_id;
+  uint32_t reserved[7];
 };
 
 grn_ra *grn_ra_create(grn_ctx *ctx, const char *path, unsigned int element_size);
@@ -58,6 +59,7 @@ grn_rc grn_ra_set_value(grn_ctx *ctx,
                         grn_id id,
                         grn_obj *value,
                         int flags);
+grn_rc grn_ra_wal_recover(grn_ctx *ctx, grn_ra *ra);
 grn_rc grn_ra_warm(grn_ctx *ctx, grn_ra *ra);
 
 typedef struct _grn_ra_cache grn_ra_cache;
@@ -87,6 +89,10 @@ struct _grn_ja {
   struct grn_ja_header *header;
 };
 
+void grn_ja_init_from_env(void);
+const char *grn_ja_segment_info_type_name(grn_ctx *ctx, uint32_t info);
+uint32_t grn_ja_segment_info_value(grn_ctx *ctx, uint32_t info);
+
 GRN_API grn_ja *grn_ja_create(grn_ctx *ctx, const char *path,
                               uint32_t max_element_size, uint32_t flags);
 grn_ja *grn_ja_open(grn_ctx *ctx, const char *path);
@@ -120,6 +126,7 @@ GRN_API grn_rc grn_ja_putv(grn_ctx *ctx, grn_ja *ja, grn_id id,
 GRN_API uint32_t grn_ja_size(grn_ctx *ctx, grn_ja *ja, grn_id id);
 
 void grn_ja_check(grn_ctx *ctx, grn_ja *ja);
+grn_rc grn_ja_wal_recover(grn_ctx *ctx, grn_ja *ja);
 grn_rc grn_ja_warm(grn_ctx *ctx, grn_ja *ra);
 
 #define GRN_JA_READER_INITIAL_REF_SEG_IDS_SIZE 16
