@@ -1,6 +1,6 @@
-/* -*- c-basic-offset: 2 -*- */
 /*
-  Copyright(C) 2010-2017 Brazil
+  Copyright(C) 2010-2017  Brazil
+  Copyright(C) 2021  Sutou Kouhei <kou@clear-code.com>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -25,15 +25,22 @@
 extern "C" {
 #endif
 
-#ifdef WIN32
+#ifdef _WIN32
+void grn_windows_init(void);
+void grn_windows_fin(void);
 GRN_API UINT grn_windows_encoding_to_code_page(grn_encoding encoding);
-bool grn_windows_symbol_initialize(HANDLE process);
-bool grn_windows_symbol_cleanup(HANDLE process);
-void grn_windows_log_trace(grn_ctx *ctx,
-                           grn_log_level level,
-                           HANDLE process,
-                           DWORD64 address);
-#endif /* WIN32 */
+bool grn_windows_symbol_initialize(grn_ctx *ctx, HANDLE process);
+bool grn_windows_symbol_cleanup(grn_ctx *ctx, HANDLE process);
+void grn_windows_log_back_trace_entry(grn_ctx *ctx,
+                                      grn_log_level level,
+                                      HANDLE process,
+                                      DWORD64 address);
+void grn_windows_log_back_trace(grn_ctx *ctx,
+                                grn_log_level level,
+                                HANDLE process,
+                                HANDLE thread,
+                                CONTEXT *context);
+#endif
 
 #ifdef __cplusplus
 }
