@@ -1433,6 +1433,8 @@ grn_index_column_is_usable(grn_ctx *ctx,
   case GRN_OP_NEAR_NO_OFFSET :
   case GRN_OP_NEAR_PHRASE :
   case GRN_OP_ORDERED_NEAR_PHRASE :
+  case GRN_OP_NEAR_PHRASE_PRODUCT :
+  case GRN_OP_ORDERED_NEAR_PHRASE_PRODUCT :
   case GRN_OP_SIMILAR :
   case GRN_OP_PREFIX :
   case GRN_OP_SUFFIX :
