@@ -309,6 +309,28 @@ ctx_is_opened(mrb_state *mrb, mrb_value self)
   return mrb_bool_value(grn_ctx_is_opened(ctx, mrb_id));
 }
 
+static mrb_value
+ctx_push_temporary_open_space(mrb_state *mrb, mrb_value self)
+{
+  grn_ctx *ctx = (grn_ctx *)mrb->ud;
+
+  grn_ctx_push_temporary_open_space(ctx);
+  grn_mrb_ctx_check(mrb);
+
+  return mrb_nil_value();
+}
+
+static mrb_value
+ctx_pop_temporary_open_space(mrb_state *mrb, mrb_value self)
+{
+  grn_ctx *ctx = (grn_ctx *)mrb->ud;
+
+  grn_ctx_pop_temporary_open_space(ctx);
+  grn_mrb_ctx_check(mrb);
+
+  return mrb_nil_value();
+}
+
 mrb_value
 grn_mrb_ctx_to_exception(mrb_state *mrb)
 {
@@ -903,5 +925,11 @@ grn_mrb_ctx_init(grn_ctx *ctx)
 
   mrb_define_method(mrb, klass, "opened?", ctx_is_opened,
                     MRB_ARGS_REQ(1));
+
+  mrb_define_method(mrb, klass, "push_temporary_open_space",
+                    ctx_push_temporary_open_space, MRB_ARGS_NONE());
+  mrb_define_method(mrb, klass, "pop_temporary_open_space",
+                    ctx_pop_temporary_open_space, MRB_ARGS_NONE());
+
 }
 #endif
