@@ -1,5 +1,8 @@
 module Groonga
   class Command
+    include Loggable
+    include QueryLoggable
+
     @@classes = {}
     class << self
       def register_class(name, klass)
@@ -20,10 +23,6 @@ module Groonga
       @writer ||= context.writer
     end
 
-    def query_logger
-      @query_logger ||= context.query_logger
-    end
-
     def cache_key(input)
       nil
     end
@@ -36,7 +35,6 @@ module Groonga
         cached_value = cache.fetch(key)
         if cached_value
           context.output = cached_value
-          cache.unref(key)
           query_logger.log(:cache, ":", "cache(#{cached_value.bytesize})")
         else
           yield
