@@ -42,9 +42,11 @@ module Groonga
       QUERY_OPERATIONS = [
         Operator::MATCH,
         Operator::NEAR,
-        Operator::NEAR2,
+        Operator::NEAR_NO_OFFSET,
         Operator::NEAR_PHRASE,
         Operator::ORDERED_NEAR_PHRASE,
+        Operator::NEAR_PHRASE_PRODUCT,
+        Operator::ORDERED_NEAR_PHRASE_PRODUCT,
         Operator::SIMILAR,
         Operator::EXACT,
         Operator::REGEXP,
