@@ -15,9 +15,11 @@ module Groonga
     RELATION_OPERATORS = [
       Operator::MATCH,
       Operator::NEAR,
-      Operator::NEAR2,
+      Operator::NEAR_NO_OFFSET,
       Operator::NEAR_PHRASE,
       Operator::ORDERED_NEAR_PHRASE,
+      Operator::NEAR_PHRASE_PRODUCT,
+      Operator::ORDERED_NEAR_PHRASE_PRODUCT,
       Operator::SIMILAR,
       Operator::PREFIX,
       Operator::SUFFIX,
@@ -84,7 +86,9 @@ module Groonga
             @data_list << data
             context.data = nil
           end
-          put_logical_op(code_op, i)
+          weight_factor = 1.0
+          weight_factor = code.value.value if code.value
+          put_logical_op(code_op, i, weight_factor)
           # TODO: rescue and return nil
           context.status = :start
         when Operator::PUSH
@@ -171,7 +175,7 @@ module Groonga
           first_data.logical_op = @operator
         end
       else
-        put_logical_op(@operator, context.n_codes)
+        put_logical_op(@operator, context.n_codes, 1)
       end
 
       optimize
@@ -269,11 +273,17 @@ module Groonga
       true
     end
 
-    def put_logical_op(operator, start)
+    def put_logical_op(operator, start, weight_factor)
       n_parens = 1
       n_dif_ops = 0
       r = 0
       j = @data_list.size
+      case operator
+      when Operator::AND_NOT, Operator::ADJUST
+        is_commutative_operator = false
+      else
+        is_commutative_operator = true
+      end
       while j > 0
         j -= 1
         data = @data_list[j]
@@ -286,7 +296,7 @@ module Groonga
             if n_parens == 0
               if r == 0
                 if n_dif_ops > 0
-                  if j > 0 and operator != Operator::AND_NOT
+                  if j > 0 and is_commutative_operator
                     n_parens = 1
                     n_dif_ops = 0
                     r = j
@@ -294,12 +304,14 @@ module Groonga
                     new_data = ScanInfoData.new(start)
                     new_data.flags = ScanInfo::Flags::POP
                     new_data.logical_op = operator
+                    new_data.weight_factor = weight_factor
                     @data_list << new_data
                     break
                   end
                 else
                   data.flags &= ~ScanInfo::Flags::PUSH
                   data.logical_op = operator
+                  data.weight_factor = weight_factor
                   break
                 end
               else
@@ -307,10 +319,12 @@ module Groonga
                   new_data = ScanInfoData.new(start)
                   new_data.flags = ScanInfo::Flags::POP
                   new_data.logical_op = operator
+                  new_data.weight_factor = weight_factor
                   @data_list << new_data
                 else
                   data.flags &= ~ScanInfo::Flags::PUSH
                   data.logical_op = operator
+                  data.weight_factor = weight_factor
                   @data_list =
                     @data_list[0...j] +
                     @data_list[r..-1] +
@@ -320,7 +334,7 @@ module Groonga
               end
             end
           else
-            if operator == Operator::AND_NOT or operator != data.logical_op
+            if !is_commutative_operator or operator != data.logical_op
               n_dif_ops += 1
             end
           end
@@ -377,7 +391,7 @@ module Groonga
             context.code_op = Operator::AND
           when Operator::OR
             @data_list[-1, 0] = create_all_match_data
-            put_logical_op(Operator::AND_NOT, i)
+            put_logical_op(Operator::AND_NOT, i, 1)
           else
             return false
           end
