@@ -16,6 +16,7 @@ module Groonga
     attr_accessor :quorum_threshold
     attr_accessor :start_position
     attr_accessor :weight
+    attr_accessor :weight_factor
     def initialize(start)
       @start = start
       @end = start
@@ -31,6 +32,7 @@ module Groonga
       @quorum_threshold = nil
       @start_position = nil
       @weight = 0
+      @weight_factor = 1.0
     end
 
     def match_resolve_index
@@ -62,10 +64,12 @@ module Groonga
     private
     def near_search?
       return true if ((@op == Operator::NEAR or
-                       @op == Operator::NEAR2) and
+                       @op == Operator::NEAR_NO_OFFSET) and
                       (@args.size == 3))
       return true if ((@op == Operator::NEAR_PHRASE or
-                       @op == Operator::ORDERED_NEAR_PHRASE) and
+                       @op == Operator::ORDERED_NEAR_PHRASE or
+                       @op == Operator::NEAR_PHRASE_PRODUCT or
+                       @op == Operator::ORDERED_NEAR_PHRASE_PRODUCT) and
                       (@args.size == 4))
       false
     end
@@ -82,7 +86,8 @@ module Groonga
       else
         message =
           "The first argument of " +
-          "NEAR/NEAR2/NEAR_PHRASE/ORDERED_NEAR_PHRASE must be " +
+          "NEAR/NEAR_NO_OFFSET/NEAR_PHRASE/ORDERED_NEAR_PHRASE/" +
+          "NEAR_PHRASE_PRODUCT/ORDERED_NEAR_PHRASE_PRODUCT must be " +
           "Expression, Accessor or Indexable: #{arg.class}"
         raise ErrorMessage, message
       end
@@ -90,7 +95,10 @@ module Groonga
       self.query = @args[1]
       self.max_interval = @args[2].value
       case @op
-      when Operator::NEAR_PHRASE, Operator::ORDERED_NEAR_PHRASE
+      when Operator::NEAR_PHRASE,
+           Operator::ORDERED_NEAR_PHRASE,
+           Operator::NEAR_PHRASE_PRODUCT,
+           Operator::ORDERED_NEAR_PHRASE_PRODUCT
         self.additional_last_interval = @args[3].value
       end
     end
