@@ -436,6 +436,20 @@ grn_snip_get_delimiter_regexp(grn_ctx *ctx, grn_obj *snip, size_t *length)
   return snip_->delimiter_pattern;
 }
 
+static grn_rc
+grn_snip_expand_cond(grn_ctx *ctx, grn_snip *snip)
+{
+  size_t new_cond_capacity = snip->cond_capacity * 2;
+  size_t new_cond_size = sizeof(snip_cond) * new_cond_capacity;
+  snip_cond *new_cond = (snip_cond *)GRN_REALLOC(snip->cond, new_cond_size);
+  if (!new_cond) {
+    return ctx->rc;
+  }
+  snip->cond = new_cond;
+  snip->cond_capacity = new_cond_capacity;
+  return GRN_SUCCESS;
+}
+
 grn_rc
 grn_snip_add_cond(grn_ctx *ctx, grn_obj *snip,
                   const char *keyword, unsigned int keyword_len,
@@ -449,13 +463,21 @@ grn_snip_add_cond(grn_ctx *ctx, grn_obj *snip,
   grn_snip *snip_;
 
   snip_ = (grn_snip *)snip;
-  if (!snip_ || !keyword || !keyword_len || snip_->cond_len >= MAX_SNIP_COND_COUNT) {
+  if (!snip_ || !keyword || !keyword_len) {
     return GRN_INVALID_ARGUMENT;
   }
 
+  if (snip_->cond_len >= snip_->cond_capacity) {
+    rc = grn_snip_expand_cond(ctx, snip_);
+    if (rc) {
+      return rc;
+    }
+  }
+
   cond = snip_->cond + snip_->cond_len;
-  if ((rc = grn_snip_cond_init(ctx, cond, keyword, keyword_len,
-                               snip_->encoding, snip_->normalizer, snip_->flags))) {
+  rc = grn_snip_cond_init(ctx, cond, keyword, keyword_len,
+                          snip_->encoding, snip_->normalizer, snip_->flags);
+  if (rc) {
     return rc;
   }
   grn_string_get_normalized(ctx, cond->keyword, NULL, &norm_blen, NULL);
@@ -583,6 +605,22 @@ grn_snip_open(grn_ctx *ctx, int flags, unsigned int width,
     GRN_API_RETURN(NULL);
   }
 
+  ret->cond_capacity = DEFAULT_SNIP_COND_CAPACITY;
+  ret->cond = (snip_cond *)GRN_MALLOC(sizeof(snip_cond) * ret->cond_capacity);
+
+  if (!ret->cond) {
+    if (copy_tag) {
+      if (ret->defaultopentag) {
+        GRN_FREE((void *)ret->defaultopentag);
+      }
+      if (ret->defaultclosetag) {
+        GRN_FREE((void *)ret->defaultclosetag);
+      }
+    }
+    GRN_FREE(ret);
+    GRN_API_RETURN(NULL);
+  }
+
   ret->cond_len = 0;
   ret->mapping = mapping;
   ret->nstr = NULL;
@@ -662,6 +700,7 @@ grn_snip_close(grn_ctx *ctx, grn_snip *snip)
        cond < cond_end; cond++) {
     grn_snip_cond_close(ctx, cond);
   }
+  GRN_FREE(snip->cond);
   GRN_FREE(snip);
   GRN_API_RETURN(GRN_SUCCESS);
 }
