@@ -48,7 +48,7 @@ grn_str_charlen_utf8(grn_ctx *ctx, const unsigned char *str, const unsigned char
   if (*str & 0x80) {
     int i;
     int len;
-    GRN_BIT_SCAN_REV(~(*str << 24), len);
+    GRN_BIT_SCAN_REV(~(((uint32_t)*str) << 24), len);
     len = 31 - len;
     if ((unsigned int)(len - 2) >= 3) {  /* (len == 1 || len >= 5) */
       GRN_LOG(ctx, GRN_LOG_WARNING,
@@ -2000,9 +2000,11 @@ grn_bulk_write(grn_ctx *ctx, grn_obj *buf, const char *str, size_t len)
     new_size = grn_bulk_compute_new_size(ctx, buf, GRN_BULK_VSIZE(buf) + len);
     if ((rc = grn_bulk_resize(ctx, buf, new_size))) { return rc; }
   }
-  curr = GRN_BULK_CURR(buf);
-  grn_memcpy(curr, str, len);
-  GRN_BULK_INCR_LEN(buf, len);
+  if (len > 0) {
+    curr = GRN_BULK_CURR(buf);
+    grn_memcpy(curr, str, len);
+    GRN_BULK_INCR_LEN(buf, len);
+  }
   return rc;
 }
 
@@ -2052,7 +2054,7 @@ grn_rc
 grn_bulk_truncate(grn_ctx *ctx, grn_obj *bulk, size_t len)
 {
   if (GRN_BULK_OUTP(bulk)) {
-    if ((bulk->u.b.tail - bulk->u.b.head) < len) {
+    if ((size_t)(bulk->u.b.tail - bulk->u.b.head) < len) {
       return grn_bulk_space_clear(ctx, bulk, len - GRN_BULK_VSIZE(bulk));
     } else {
       if (GRN_BULK_VSIZE(bulk) < len) {
