@@ -1478,6 +1478,8 @@ select_index_dispatch(grn_ctx *ctx,
   case GRN_OP_NEAR_NO_OFFSET :
   case GRN_OP_NEAR_PHRASE :
   case GRN_OP_ORDERED_NEAR_PHRASE :
+  case GRN_OP_NEAR_PHRASE_PRODUCT :
+  case GRN_OP_ORDERED_NEAR_PHRASE_PRODUCT :
   case GRN_OP_SIMILAR :
   case GRN_OP_REGEXP :
   case GRN_OP_QUORUM :
@@ -1636,6 +1638,7 @@ select_index(grn_ctx *ctx,
                                    si->logical_op,
                                    table_selector);
       }
+      rc = grn_ctx_rc_propagate(ctx, rc);
       if (rc != GRN_SUCCESS) {
         break;
       }
@@ -1717,6 +1720,7 @@ select_index(grn_ctx *ctx,
     }
   }
 
+  rc = grn_ctx_rc_propagate(ctx, rc);
   switch (rc) {
   case GRN_SUCCESS :
     return true;
@@ -1838,6 +1842,8 @@ inspect_condition(grn_ctx *ctx,
   case GRN_OP_NEAR_NO_OFFSET :
   case GRN_OP_NEAR_PHRASE :
   case GRN_OP_ORDERED_NEAR_PHRASE :
+  case GRN_OP_NEAR_PHRASE_PRODUCT :
+  case GRN_OP_ORDERED_NEAR_PHRASE_PRODUCT :
   case GRN_OP_PREFIX :
     if (n_codes == 3) {
       grn_expr_code *arg1 = expr->codes + si->start;
@@ -2091,10 +2097,7 @@ grn_table_selector_select(grn_ctx *ctx,
           data->min_id = table_selector->min_id;
         }
         processed = select_index(ctx, table_selector);
-        if (!processed) {
-          if (ctx->rc != GRN_SUCCESS) {
-            break;
-          }
+        if (!processed && ctx->rc == GRN_SUCCESS) {
           e->codes = codes + si->start;
           e->codes_curr = si->end - si->start + 1;
           grn_table_selector sequential_table_selector;
