@@ -1,4 +1,3 @@
-/* -*- c-basic-offset: 2 -*- */
 /*
   Copyright(C) 2012-2018  Brazil
   Copyright(C) 2018-2020  Sutou Kouhei <kou@clear-code.com>
@@ -176,6 +175,8 @@ grn_tokenizer_query_init(grn_ctx *ctx, grn_tokenizer_query *query)
   query->need_normalize = GRN_TRUE;
   query->need_delimiter_check = GRN_TRUE;
 
+  query->options = NULL;
+
   return ctx->rc;
 }
 
@@ -504,6 +505,24 @@ grn_tokenizer_query_set_index_column(grn_ctx *ctx,
   GRN_API_RETURN(ctx->rc);
 }
 
+grn_obj *
+grn_tokenizer_query_get_options(grn_ctx *ctx,
+                                grn_tokenizer_query *query)
+{
+  GRN_API_ENTER;
+  GRN_API_RETURN(query->options);
+}
+
+grn_rc
+grn_tokenizer_query_set_options(grn_ctx *ctx,
+                                grn_tokenizer_query *query,
+                                grn_obj *options)
+{
+  GRN_API_ENTER;
+  query->options = options;
+  GRN_API_RETURN(ctx->rc);
+}
+
 void
 grn_tokenizer_token_init(grn_ctx *ctx, grn_tokenizer_token *token)
 {
