@@ -1,7 +1,6 @@
-/* -*- c-basic-offset: 2 -*- */
 /*
   Copyright(C) 2018  Brazil
-  Copyright(C) 2020  Sutou Kouhei <kou@clear-code.com>
+  Copyright(C) 2020-2021  Sutou Kouhei <kou@clear-code.com>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -20,6 +19,7 @@
 #include "grn.h"
 #include "grn_ctx.h"
 #include "grn_float.h"
+#include "grn_io.h"
 #include "grn_vector.h"
 
 #include <cstring>
@@ -193,6 +193,24 @@ extern "C" {
     return grn::vector_get_element<uint64_t>(ctx, vector, offset, default_value);
   }
 
+  float
+  grn_vector_get_element_float32(grn_ctx *ctx,
+                                 grn_obj *vector,
+                                 uint32_t offset,
+                                 float default_value)
+  {
+    return grn::vector_get_element<float>(ctx, vector, offset, default_value);
+  }
+
+  double
+  grn_vector_get_element_float64(grn_ctx *ctx,
+                                 grn_obj *vector,
+                                 uint32_t offset,
+                                 double default_value)
+  {
+    return grn::vector_get_element<double>(ctx, vector, offset, default_value);
+  }
+
   static uint32_t
   grn_uvector_element_size_internal(grn_ctx *ctx, grn_obj *uvector)
   {
@@ -704,4 +722,34 @@ extern "C" {
   exit :
     GRN_API_RETURN(id);
   }
+
+  grn_rc
+  grn_uvector_copy(grn_ctx *ctx,
+                   grn_obj *src,
+                   grn_obj *dest)
+  {
+    GRN_API_ENTER;
+    if (src->header.domain != dest->header.domain) {
+      ERR(GRN_INVALID_ARGUMENT,
+          "[uvector][copy] different domain: "
+          "source=<%u> "
+          "destination=<%u>",
+          src->header.domain,
+          dest->header.domain);
+      GRN_API_RETURN(ctx->rc);
+    }
+    bool src_have_weight = grn_obj_is_weight_uvector(ctx, src);
+    bool dest_have_weight = grn_obj_is_weight_uvector(ctx, dest);
+    if (src_have_weight != dest_have_weight) {
+      ERR(GRN_INVALID_ARGUMENT,
+          "[uvector][copy] different weight availability: "
+          "source=<%s> "
+          "destination=<%s>",
+          src_have_weight ? "true" : "false",
+          dest_have_weight ? "true" : "false");
+      GRN_API_RETURN(ctx->rc);
+    }
+    grn_bulk_write(ctx, dest, GRN_BULK_HEAD(src), GRN_BULK_VSIZE(src));
+    GRN_API_RETURN(ctx->rc);
+  }
 }
