@@ -1,4 +1,3 @@
-/* -*- c-basic-offset: 2 -*- */
 /* Copyright(C) 2012-2015 Brazil
 
   This library is free software; you can redistribute it and/or
@@ -47,7 +46,7 @@ get_system_synonyms_file(void)
     const char *relative_path = GRN_QUERY_EXPANDER_TSV_RELATIVE_SYNONYMS_FILE;
     size_t base_dir_length;
 
-    base_dir = grn_plugin_win32_base_dir();
+    base_dir = grn_plugin_windows_base_dir();
     base_dir_length = strlen(base_dir);
     grn_strcpy(win32_synonyms_file, MAX_PATH, base_dir);
     grn_strcat(win32_synonyms_file, MAX_PATH, "/");
