@@ -112,12 +112,12 @@ stop_word_init(grn_ctx *ctx, grn_tokenizer_query *query)
 
   lexicon = grn_tokenizer_query_get_lexicon(ctx, query);
   i = grn_tokenizer_query_get_token_filter_index(ctx, query);
-  options = grn_table_cache_token_filter_options(ctx,
-                                                 lexicon,
-                                                 i,
-                                                 stop_word_open_options,
-                                                 stop_word_close_options,
-                                                 NULL);
+  options = grn_table_cache_token_filters_options(ctx,
+                                                  lexicon,
+                                                  i,
+                                                  stop_word_open_options,
+                                                  stop_word_close_options,
+                                                  NULL);
   if (ctx->rc != GRN_SUCCESS) {
     return NULL;
   }
