@@ -1,4 +1,3 @@
-/* -*- c-basic-offset: 2; coding: utf-8 -*- */
 /*
   Copyright (C) 2009-2011  Kouhei Sutou <kou@clear-code.com>
 
@@ -13,10 +12,10 @@
 
   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free Software
-  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
-#include <db.h>
+#include <grn_db.h>
 
 #include <gcutter.h>
 #include <glib/gstdio.h>
@@ -50,6 +49,8 @@ void test_text_to_geo_point_mixed_invalid(void);
 void data_text_error(void);
 void test_text_error(gconstpointer data);
 
+void test_bool_to_bool(void);
+
 void data_int32_to_bool(void);
 void test_int32_to_bool(gconstpointer data);
 void test_int32_to_int8(void);
@@ -410,6 +411,28 @@ test_text_error(gconstpointer data)
                            grn_obj_cast(&context, &src, &dest, GRN_FALSE));
 }
 
+static void
+set_bool(gboolean boolean)
+{
+  grn_obj_reinit(&context, &src, GRN_DB_BOOL, 0);
+  GRN_BOOL_SET(&context, &src, boolean);
+}
+
+static void
+cast_bool(gboolean boolean)
+{
+  set_bool(boolean);
+  grn_test_assert(grn_obj_cast(&context, &src, &dest, GRN_FALSE));
+}
+
+void
+test_bool_to_bool(void)
+{
+  grn_obj_reinit(&context, &dest, GRN_DB_BOOL, 0);
+  cast_bool(TRUE);
+  cut_assert_true(GRN_BOOL_VALUE(&dest));
+}
+
 static void
 set_int32(gint32 number)
 {
